package missing;

import java.awt.BorderLayout;
import java.io.*;
import java.util.Set;

import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;

import bibliothek.gui.DockFrontend;
import bibliothek.gui.Dockable;
import bibliothek.gui.dock.SplitDockStation;
import bibliothek.gui.dock.facile.menu.CloseableDockableMenuPiece;
import bibliothek.gui.dock.facile.menu.FrontendSettingsMenuPiece;
import bibliothek.gui.dock.facile.menu.RootMenuPiece;
import bibliothek.gui.dock.frontend.MissingDockableStrategy;
import bibliothek.util.xml.XElement;
import bibliothek.util.xml.XIO;

public class MainFrame extends JFrame {
    private DockFrontend frontend;
    private boolean factoryHandled = false;
    
    public MainFrame(){
        setTitle( "Missing Dockable Test" );
        
        frontend = new DockFrontend( this );
        frontend.setMissingDockableStrategy( MissingDockableStrategy.STORE_ALL );
        frontend.setDefaultEntryLayout( false );
        frontend.setDefaultHideable( true );
        
        SplitDockStation root = new SplitDockStation();
        add( root, BorderLayout.CENTER );
        
        frontend.addRoot( "root", root );
        
        MainMenu menu = new MainMenu( this );
        JMenuBar bar = new JMenuBar();
        
        RootMenuPiece frontendMenu = new RootMenuPiece( "Layout", true, new FrontendSettingsMenuPiece( frontend, false ) );
        RootMenuPiece panelMenu = new RootMenuPiece( "Dockables", true, new CloseableDockableMenuPiece( frontend ));
        
        bar.add( menu );
        bar.add( frontendMenu.getMenu() );
        bar.add( panelMenu.getMenu() );
        
        setJMenuBar( bar );
    }
    
    public void add( String id, Dockable dockable ){
        if( !factoryHandled ){
            factoryHandled = true;
            frontend.registerFactory( TestDockable.FACTORY );
        }
        
        frontend.addDockable( id, dockable );
        frontend.show( dockable );
    }
    
    public String[] listEmptyIds(){
        return frontend.listEmpty( false );
    }
    
    /**
     * Gets a list of all ids that are in use and must not be
     * used by another {@link Dockable}.
     * @return the list of ids
     */
    public String[] listUsedIds(){
        Set<String> set = frontend.getNamedDockables().keySet();
        return set.toArray( new String[ set.size() ] );
    }
    
    public void writeXML(){
        try{
            XElement root = new XElement( "root" );
            frontend.writeXML( root );
            FileOutputStream out = new FileOutputStream( "layout.xml" );
            XIO.writeUTF( root, out );
            out.close();
        }
        catch( Throwable t ){
            JOptionPane.showMessageDialog( this, "Cannot store:\n" + t.getMessage(), "Error", JOptionPane.ERROR_MESSAGE );
            t.printStackTrace();
        }
    }
    
    public void readXML(){
        try{
            FileInputStream in = new FileInputStream( "layout.xml" );
            XElement root = XIO.readUTF( in );
            in.close();
            frontend.readXML( root );
        }
        catch( Throwable t ){
            JOptionPane.showMessageDialog( this, "Cannot read:\n" + t.getMessage(), "Error", JOptionPane.ERROR_MESSAGE );
            t.printStackTrace();
        }
    }
    
    public void writeStream(){
        try{
            DataOutputStream out = new DataOutputStream( new FileOutputStream( "layout.stream" ));
            frontend.write( out );
            out.close();
        }
        catch( Throwable t ){
            JOptionPane.showMessageDialog( this, "Cannot store:\n" + t.getMessage(), "Error", JOptionPane.ERROR_MESSAGE );
            t.printStackTrace();
        }        
    }
    
    public void readStream(){
        DataInputStream in = null;
        try{
            in = new DataInputStream( new FileInputStream( "layout.stream" ) );
            frontend.read( in );
        }
        catch( Throwable t ){
            try {
                if( in != null ){
                    in.close();
                }
            }
            catch( IOException e ) {
                e.printStackTrace();
            }
            JOptionPane.showMessageDialog( this, "Cannot load:\n" + t.getMessage(), "Error", JOptionPane.ERROR_MESSAGE );
            t.printStackTrace();
        }        
    }
}
